@@ -9,7 +9,7 @@ use Net::LDAP::ASN qw(LDAPRequest);
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "1.10";
+$VERSION = "1.11";
 
 my $MsgID = 0;
 
@@ -102,10 +102,12 @@ sub set_error {
 }
 
 sub error_name {
+  require Net::LDAP::Util;
   Net::LDAP::Util::ldap_error_name(shift->code);
 }
 
 sub error_text {
+  require Net::LDAP::Util;
   Net::LDAP::Util::ldap_error_text(shift->code);
 }
 
@@ -142,9 +144,21 @@ sub decode { # $self, $pdu, $control
   # free up memory as we have a result so we will not need to re-send it
   delete $self->{pdu};
 
-  # tell our LDAP client to forget us as this message has now completed
-  # all communications with the server
-  $self->parent->_forgetmesg($self);
+  if ($data = delete $result->{protocolOp}{intermediateResponse}) {
+
+    my $intermediate = Net::LDAP::Intermediate->from_asn($data);
+
+    push(@{$self->{'intermediate'} ||= []}, $intermediate);
+
+    $self->{callback}->($self, $intermediate)
+      if (defined $self->{callback});
+
+    return $self;
+  } else {
+    # tell our LDAP client to forget us as this message has now completed
+    # all communications with the server
+    $self->parent->_forgetmesg($self);
+  }
 
   $self->{callback}->($self)
     if (defined $self->{callback});
@@ -196,12 +210,15 @@ sub control {
     }
   }
 
-  return unless $self->{ctrl_hash};
+  my $ctrl_hash = $self->{ctrl_hash}
+    or return;
 
-  @_ ?  exists $self->{ctrl_hash}{$_[0]}
-         ? @{$self->{ctrl_hash}{$_[0]}}
-         : ()
-     : map { @$_ } values %{$self->{ctrl_hash}};
+  my @oid = @_ ? @_ : keys %$ctrl_hash;
+  my @control = map {@$_} grep $_, @{$ctrl_hash}{@oid}
+    or return;
+
+  # return a list, so in a scalar context we do not just get array length
+  return @control[0 .. $#control];
 }
 
 sub pdu      {  shift->{pdu}      }
